/************************************************************************
*
*		   TCA SCA Window Register Calculation Program
*			Product # 48-0913
*
*	This program is used to calculate the register values to program
*	the three SCA windows.  
*
*	Local symbols are returned to indicate the values that must be
*	set in the icb registers. 
*
*	Calling Format:
*
*	ND_TCA_CALCSCAREG <config> <window>
*
*	Where:
*		<config> is the input configuration setup
*		<channel> is the channel number 
*		<window> is the window number (from 1 to 3)
*
*************************************************************************
*
*	Revision History:
*
*	08-DEC-1996	ARW	Original version
*
************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <descrip.h>
#include <ndtypes.h>
#include <camdef.h>
#include <campardef.h>
#include <cammisc.h>
#include <cammsgdef.h>
#include <starlet.h>
#include <lib$routines.h>
#include <libclidef.h>

/* error handling macro */
#define IFERROR(status) if(!(status&1))

/* define a nearest integer rounding function */
#define nint(a) (((a) < 0) ? (int) ((a) - 0.5) : (int) ((a) + 0.5))

/* local functions */
struct dsc$descriptor_s createDescrip(char *s);
int setSymbol(char *name, char *value);

/* external functions */
extern cam_open(); 
extern cam_close(); 
extern cam_getp_l(); 

/************************************************************************/
main(int argc, char *argv[])
{

LONG s;          			/* return status */
LONG chan; 				/* channel number */
LONG flags; 				/* cam flags */
LONG window = 0; 			/* window number */
LONG channel = 0; 			/* channel number */
LONG surint2; 				/* energy range */
REAL surreala;				/* centroids */
REAL surrealb; 				/* widths */
REAL width; 	
REAL centroid; 
REAL ftemp; 
LONG lld; 
LONG uld;  
LONG lolld, hilld, hilldreg, lould, hiuld, hiuldreg; 
char temp[256]; 
struct dsc$descriptor_s camfile;	/* cam file descriptor */
struct cam_plist_struct *plist = NULL;  /* generic plist pointer */

/* the special uses record number */
struct cam_plist_struct plist1[] = {
        { CAM_L_SURINT2,        0, NULL },
        { 0,                    0, NULL }
};

/* items for each window */
struct cam_plist_struct plist2[] = {
        { CAM_F_SURREAL1,        0, NULL },
        { CAM_F_SURREAL4,        0, NULL },
        { 0,                     0, NULL }
};

struct cam_plist_struct plist3[] = {
        { CAM_F_SURREAL2,        0, NULL },
        { CAM_F_SURREAL5,        0, NULL },
        { 0,                     0, NULL }
};

struct cam_plist_struct plist4[] = {
        { CAM_F_SURREAL3,        0, NULL },
        { CAM_F_SURREAL6,        0, NULL },
        { 0,                     0, NULL }
};

plist1[0].address = &surint2;		/* energy range */

plist2[0].address = &surreala;
plist2[1].address = &surrealb;

plist3[0].address = &surreala;
plist3[1].address = &surrealb;

plist4[0].address = &surreala;
plist4[1].address = &surrealb;

	/* make sure the program was called correctly */
	if (argc < 3)
	{
		fprintf(stdout, "Usage: ND_TCA_CALCSCAREG <config> <channel> <window>\n"); 
		exit(1); 
	}

	/* get the channel number */
	channel = atoi(argv[2]); 

	/* get the window number */
	window = atoi(argv[3]); 

	/* exit if the window number is not valid, use a generic message */
	if ((window < 1) || (window > 3))
	{
		exit(CAM__INVALREC); 
	}

	/* set up the record plist to use the correct cam parameter list */
	if (window == 1)
	{
		plist = plist2; 
	}
	else if (window == 2)
	{
		plist = plist3; 
	}
	else 
	{
		plist = plist4; 
	}

	/* create a descriptor with the cam file name */
	camfile = createDescrip(argv[1]); 

	/* open the cam file with the setup information */
	flags = CAM_M_COC_RONLY; 
	s = cam_open(&chan, &camfile, &flags); 
	IFERROR(s) exit(s); 

	/* get the energy range */
        flags = 0; 
        s = cam_getp_l(&chan, plist1, &channel, 0, &flags);
        IFERROR(s) exit(s); 

	/* get the window settings */
        flags = 0; 
        s = cam_getp_l(&chan, plist, &channel, 0, &flags);
        IFERROR(s) exit(s); 

	/* close the input cam file */
	s = cam_close(&chan); 
	IFERROR(s) exit(s); 

	/* perform the calculations */
	width = (surrealb / surint2) * 4096.0; 
	centroid = (surreala / surint2) * 4096.0; 
	ftemp = centroid - (width / 2.0); 
	lld = nint(ftemp); 
	ftemp = centroid + (width / 2.0); 
	uld = nint(ftemp); 
	if (lld <= 0) lld = 1; 
	if (uld >= 4096) uld = 4095;		

	/* get the register values */
	lolld    = lld & 0xFF; 
	hilld    = (lld & 0xF00) >> 8; 
	hilldreg = hilld + (64 * (window - 1));
	lould    = uld & 0xFF;
	hiuld    = (uld & 0xF00) >> 8; 
	hiuldreg = hiuld + 32 + (64 * (window - 1));

	/* return the symbols required by the command procedure */
	sprintf(temp, "%d", lolld); 
	s = setSymbol("LOLLD", temp); 
	IFERROR(s) exit(s); 

	sprintf(temp, "%d", hilldreg);
	s = setSymbol("HILLDREG", temp); 
	IFERROR(s) exit(s); 

	sprintf(temp, "%d", lould); 
	s = setSymbol("LOULD", temp); 
	IFERROR(s) exit(s); 
	
	sprintf(temp, "%d", hiuldreg);
	s = setSymbol("HIULDREG", temp); 
	IFERROR(s) exit(s); 
	
	/* return success */
	return(1); 
} 

/************************************************************************
*
* Function: createDescrip
*
* Description: 	This function will return a string descriptor properly 
* 		initialized to point to the null terminated string that
*		is passed in as an argument. 
*
* Usage:
*
*	descrip = createDescrip(string)
*
*	where:
*
*		descrip = returned string descriptor structure
*       	string  = a character pointer to a null terminated string
*	
************************************************************************/

struct dsc$descriptor_s createDescrip(char *s)
{

struct dsc$descriptor_s temp; 

	temp.dsc$b_class = DSC$K_CLASS_S; 	/* string descriptor */
	temp.dsc$b_dtype  = DSC$K_DTYPE_T; 	/* text data type */
	temp.dsc$a_pointer = s; 		/* string address */
	temp.dsc$w_length = strlen(s); 		/* length of string */

	/* return the initialized string descriptor */
	return(temp); 

}

/************************************************************************
*
* Function: setSymbol
*
* Description: 	This function will set an individual DCL symbol. 
*
* Usage:
*
*	s = setSymbol(name, value)
*
*	where:
*
*		s = return status
*
*		name = symbol name to set
*
*		value = string to set the symbol
*	
************************************************************************/

int setSymbol(char *name, char *value)
{

struct dsc$descriptor_s temp1; 
struct dsc$descriptor_s temp2; 
int table = LIB$K_CLI_LOCAL_SYM; 


	/* create descriptors for the name and value */
	temp1 = createDescrip(name); 
	temp2 = createDescrip(value); 

	/* call the function to set up the global symbol */
	return(lib$set_symbol(&temp1, &temp2, &table)); 

}

/* end of file */

