$ on warning then goto handle_error
$ on control_y then goto handle_error
$!
$!****************************************************************************
$!
$!	48-0913 AMP/TCA 2016 Calibration Procedure
$!
$! This procedure is used to perform calibration on the 2016
$! module. 
$!
$! Revision History:
$!
$!	21-NOV-1996	ARW	Original
$!
$!****************************************************************************
$!
$! Initialize the last tca module information that was selected variables. 
$ last_channel   == "0"
$ last_reg2value ==  0
$ last_icbnum    ==  0
$ last_aimname   ==  0
$!
$! See if detector one exists, if not show the user an error message.
$ if f$trnlnm("ACQSTATE_''username'_DET01","ND_MCA_SYSTEM_TABLE") .eqs. ""
$ then
$    ND_SA$POP_MSG_ACKN "''txt_det1notavail'"
$    goto finished
$ endif
$!
$! Call the routine to check out the hardware modules. 
$ @nd_0913_com:nd_0913_checkmods.com 
$! 
$! Clear the cc nim errors. 
$ pars/command mca0:['username']det01 astfbadcal=0 astfccerr=0 ! astfccfemod=0
$!
$! Put up the MVC window if it is not already up. 
$ ND_SA$CREATE_DISPLAY 1
$!
$! Clear the energy calibration information for detector one. 
$ pars/command mca0:['username']det01 -
        ecoffset="0" -
        ecslope="0" -
        ecalterms="0" -
        ecaltype="" -
        ecalunits="" -
        ecalcnv="0"
$!
$! Display detector one on the mca view control.
$ set noon
$    display/config mca0:['username']det01
$ set on
$!
$! Do the same for detector two if it exists. 
$ if f$trnlnm("ACQSTATE_''username'_DET02","ND_MCA_SYSTEM_TABLE") .nes. ""
$ then 
$   pars/command mca0:['username']det02 astfbadcal=0 astfccerr=0 ! astfccfemod=0
$   pars/command mca0:['username']det02 -
        ecoffset="0" -
        ecslope="0" -
        ecalterms="0" -
        ecaltype="" -
        ecalunits="" -
        ecalcnv="0"
$ endif
$!
$! If the channel number is not set yet, then default to 1. 
$ pars/command/read nd_0913_setup:setup sucint1
$ if (f$integer(sucint1) .lt. 1) .or. (f$integer(sucint1) .gt. 32)
$ then 
$    pars/command nd_0913_setup:setup sucint1=1  
$ endif
$!
$gainloop1:
$!
$! Deselect the last TCA that was selected so that only one tca is ever
$! selected at a particular time. 
$  if (last_channel .nes. "0") 
$  then
$!
$!     Deselect the last channel. 
$      mca/icb 'last_aimname':'f$fao("!1XL",last_icbnum)' 2/sym=new_reg2val
$      select_off = f$integer(new_reg2val) .and. 127
$      mca/icb 'last_aimname':'f$fao("!1XL",last_icbnum)' 2="''select_off'"
$!
$  endif
$!
$! Pop up the dialog to allow the user to select the channel. 
$  ND_SA$PARS "nd_0913_setup:setup nd_0913_text:channel_choice.fpl"
$!
$! If the user pressed cancel, then simply return. 
$ if ND_SA_REPLY .eqs. CP_REPLY_CANCEL then goto finished
$!
$! Get the user's choice of channel. 
$ pars/command/read nd_0913_setup:setup.cnf sucint1
$!
$! See if this channel is active or not. 
$ channel = f$integer(sucint1)
$ if (chnactive_'f$fao("!2ZL",channel)' .eq. 0)
$ then 
$    ND_SA$POP_MSG_ACKN "''txt_offline'"
$    goto gainloop1
$ endif
$!
$! Open the proper input (1 or 2) and prepare for calibration. 
$ newinputnum == ((channel - 1) / 16) + 1 
$!
$! Let the user know that acquisition is starting. 
$ ND_SA$POP_MSG "''txt_startacq'1"
$!
$! Wipe out the existing calibration before starting.
$ pars/command mca0:['username']det01 -
        ecoffset="0" -
        ecslope="0" -
        ecalterms="0" -
        ecaltype="" -
        ecalunits="" -
        ecalcnv="0"
$!
$! Assert the select TCA line for the channel chosen. 
$ if (f$integer(newinputnum) .eq. 1)
$ then
$    icbnum = f$integer(channel) - 1
$ else
$    icbnum = f$integer(channel) - 17
$ endif
$ pars/command/read mca0:['username']det0'newinputnum' cpname
$ aimname = f$extract(0,f$locate(":",cpname),cpname)
$ mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2/sym=new_reg2val
$ select_on = f$integer(new_reg2val) .or. 128
$ mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2="''select_on'"
$!
$! Start acquisition on the selected mca configuration. 
$ pars/command mca0:['username']det01 plive=0 preal=0
$ mca/command "erase mca0:[''username']det01"
$ mca/command "on    mca0:[''username']det01"
$!
$! Pull out all of the information required for this record (sucint1)
$! and set up local job logicals that will be used to populate the 
$! data entry dialog. 
$ pars/command/read/record='sucint1' nd_0913_setup:setup.cnf -
		surflag0 -
		surflag1 -
		surint2  -
		surint3  - 
		surint4  
$ pars/command/read/record='sucint1' nd_0913_setup:setup.cnf -
		surflag2 - 
		surflag3 - 
		surreal1 - 
		surreal2 - 
		surreal3 - 
		surreal4 
$ pars/command/read/record='sucint1' nd_0913_setup:setup.cnf -
		surreal5 - 
		surreal6 -
		surflag4 - 
		surflag5 - 
		surflag6
$ pars/command/read nd_0913_setup:setup.cnf -
		sucflag0 
$!
$! Make sure the energy range is not 0.
$ energyrange = f$integer(surint2)
$ if (energyrange .eq. 0) then surint2 = 10
$!
$! Based on the actual parameter values, set up the logicals used in 
$! the fpl that is shown to the user. 
$!
$  if (f$edit(surflag0,"TRIM,COMPRESS,UPCASE").eqs."YES")
$  then 
$     define/job/nolog nd_pur        "Enabled"
$  else
$     define/job/nolog nd_pur        "Disabled"
$  endif 
$!
$  define/job/nolog nd_energy "''surint2'"
$!
$  if (f$edit(surint3,"TRIM,COMPRESS,UPCASE").eqs."1")
$  then 
$     define/job/nolog nd_tca        "Normal"
$  else
$     define/job/nolog nd_tca        "Inverted"
$  endif
$!
$  if (f$edit(sucflag0,"TRIM,COMPRESS,UPCASE").eqs."YES")
$  then 
$     define/job/nolog nd_apply      "1"
$  else
$     define/job/nolog nd_apply      "0"
$  endif
$!
$  if (f$edit(surflag2,"TRIM,COMPRESS,UPCASE").eqs."YES")
$  then 
$     define/job/nolog nd_purauto    "Enabled"
$  else
$     define/job/nolog nd_purauto    "Disabled"
$  endif
$!
$  define/job/nolog nd_centroid1  "''surreal1'"
$  define/job/nolog nd_centroid2  "''surreal2'"
$  define/job/nolog nd_centroid3  "''surreal3'"
$!
$  define/job/nolog nd_width1     "''surreal4'"
$  define/job/nolog nd_width2     "''surreal5'"
$  define/job/nolog nd_width3     "''surreal6'"
$!
$  if (f$edit(surflag4,"TRIM,COMPRESS,UPCASE").eqs."YES")
$  then 
$     define/job/nolog nd_outenable1 "1"
$  else
$     define/job/nolog nd_outenable1 "0"
$  endif
$!
$  if (f$edit(surflag5,"TRIM,COMPRESS,UPCASE").eqs."YES")
$  then 
$     define/job/nolog nd_outenable2 "1"
$  else
$     define/job/nolog nd_outenable2 "0"
$  endif
$!
$  if (f$edit(surflag6,"TRIM,COMPRESS,UPCASE").eqs."YES")
$  then 
$     define/job/nolog nd_outenable3 "1"
$  else
$     define/job/nolog nd_outenable3 "0"
$  endif
$!
$ define/job/nolog nd_module "''sucint1'"
$!
$gainloop2:
$!
$! Show the calibration dialog. 
$ ND_SA$PARS "nd_0913_setup:setup nd_0913_text:gain_setup.fpl"
$!
$! Interpret the results and put everything back into the 
$! setup configuration file for this record. 
$!
$  if (f$edit(f$trnlnm("ND_PUR","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."ENABLED")
$  then
$       surflag0 = 1
$	surflag1 = 0
$  else
$       surflag0 = 0
$	surflag1 = 1
$  endif
$!
$  surint2 = "''f$trnlnm("ND_ENERGY","lnm$job")"
$!
$  if (f$edit(f$trnlnm("ND_TCA","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."NORMAL")
$  then 
$     surint3 = 1
$     surint4 = 0
$  else
$     surint3 = 0
$     surint4 = 1
$  endif
$!
$  if (f$edit(f$trnlnm("ND_APPLY","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."1")
$  then 
$     sucflag0 = 1
$  else
$     sucflag0 = 0
$  endif
$!
$  if (f$edit(f$trnlnm("ND_PURAUTO","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."ENABLED")
$  then 
$     surflag2 = 1
$     surflag3 = 0
$  else
$     surflag2 = 0
$     surflag3 = 1
$  endif
$!
$  surreal1 = "''f$trnlnm("ND_CENTROID1","lnm$job")'"
$  surreal2 = "''f$trnlnm("ND_CENTROID2","lnm$job")'"
$  surreal3 = "''f$trnlnm("ND_CENTROID3","lnm$job")'"
$!
$  surreal4 = "''f$trnlnm("ND_WIDTH1","lnm$job")'"
$  surreal5 = "''f$trnlnm("ND_WIDTH2","lnm$job")'"
$  surreal6 = "''f$trnlnm("ND_WIDTH3","lnm$job")'"
$!
$  if (f$edit(f$trnlnm("ND_OUTENABLE1","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."1")
$  then 
$     surflag4 = 1
$  else
$     surflag4 = 0
$  endif
$!
$  if (f$edit(f$trnlnm("ND_OUTENABLE2","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."1")
$  then 
$     surflag5 = 1
$  else
$     surflag5 = 0
$  endif
$!
$  if (f$edit(f$trnlnm("ND_OUTENABLE3","lnm$job"),-
                 "TRIM,COMPRESS,UPCASE").eqs."1")
$  then 
$     surflag6 = 1
$  else
$     surflag6 = 0
$  endif
$!
$! Move everything back into the setup configuration file. 
$!
$ pars/command/record='sucint1' nd_0913_setup:setup.cnf -
		surflag0="''surflag0'" -
		surflag1="''surflag1'" -
		surint2="''surint2'"  -
		surint3="''surint3'"  - 
		surint4="''surint4'"  
$ pars/command/record='sucint1' nd_0913_setup:setup.cnf -
		surflag2="''surflag2'" - 
		surflag3="''surflag3'" - 
		surreal1="''surreal1'" - 
		surreal2="''surreal2'" - 
		surreal3="''surreal3'" - 
		surreal4="''surreal4'"
$ pars/command/record='sucint1' nd_0913_setup:setup.cnf -
		surreal5="''surreal5'" - 
		surreal6="''surreal6'" -
		surflag4="''surflag4'" - 
		surflag5="''surflag5'" - 
		surflag6="''surflag6'"
$ pars/command nd_0913_setup:setup.cnf -
		sucflag0="''sucflag0'"
$!
$! Check for valid centroid/width values (they must not exceed
$! the energy range entered for this record).  This program returns
$! the local symbol ENERGYCHECK for testing. 
$ nd_tca_checkenergy :== $nd_sysexe:nd_tca_checkenergy.exe
$ nd_tca_checkenergy nd_0913_setup:setup.cnf
$!
$ if energycheck .eqs. "BAD" 
$ then 
$    ND_SA$POP_MSG_ACKN "''txt_badvalues'"
$    goto gainloop2
$ endif
$!
$! Call the routine to program the module number that was selected. 
$ ND_SA$POP_MSG "''txt_progtca' ''sucint1'"
$ @nd_0913_com:nd_0913_progtca.com 'newinputnum' 'sucint1' 1 1 'sucflag0'
$!
$! Check the return status of the programming operation. 
$ if (ND_PROGTCASTATUS .eqs. "ERROR") 
$ then 
$    ND_SA$POP_MSG "''txt_stopacq'1"
$    mca/command "off    mca0:[''username']det01"
$    goto gainloop1
$ endif
$!
$! Assert the select TCA line for the last channel chosen. 
$  if (last_channel .nes. "0") 
$  then
$     mca/icb 'last_aimname':'f$fao("!1XL",last_icbnum)' 2/sym=new_reg2val
$     select_on = f$integer(new_reg2val) .or. 128
$     mca/icb 'last_aimname':'f$fao("!1XL",last_icbnum)' 2="''select_on'"
$  endif
$!
$! Clear the mca data and restart acquisition. 
$ pars/command mca0:['username']det01 plive=0 preal=0
$ mca/command "erase mca0:[''username']det01"
$ mca/command "on    mca0:[''username']det01"
$!
$! Ask the user if he wants to calibrate or not. 
$ ND_SA$POP_CONFIRM "''txt_contadjust'"
$!
$! If the user selected calibrate, then perform a calibration. 
$ if ND_SA_REPLY .eqs. CP_REPLY_CANCEL 
$ then 
$!
$!   Perform the calibration and transfer it to the detector. 
$    ND_SA$PARS "nd_0913_setup:setup nd_0913_text:cal_choice.fpl"
$!
$!   If the user did not press cancel then do the calibration.  Otherwise
$!   stop the acquisition and continue below. 
$    if ND_SA_REPLY .nes. CP_REPLY_CANCEL 
$    then 
$!
$!	Get the cursor position (channel) the user selected. 
$       pars/command/read mca0:['username']det01 cursorch
$!
$!	Get the SCA number and energy for this position. 
$  	pars/command/read/record='sucint1' nd_0913_setup:setup.cnf -
		sucint2 -
		surreal1 -
		surreal2 - 
		surreal3
$!
$!	Get the energy, for the chosen sca. 
$	energy = surreal'sucint2'
$!
$!	Calculate the energy calibration information. 
$	nd_tca_setecal :== $nd_sysexe:nd_tca_setecal.exe
$	nd_tca_setecal 'cursorch' 'energy'
$!
$! 	Set the energy calibration information for the detector. 
$ 	pars/command mca0:['username']det01 -
	        ecoffset="''ecoffset'" -
	        ecslope="''ecslope'" -
	        ecalterms="''ecalterms'" -
	        ecaltype="''ecaltype'" -
	        ecalunits="''ecalunits'" -
	        ecalcnv="''ecalcnv'"
$!
$!	Store the energy calibration slope for this module in the 
$!	system setup file if the user says ok. 
$       ND_SA$POP_CONFIRM "''txt_calibok'"
$       if ND_SA_REPLY .NES. CP_REPLY_CANCEL
$	then 
$  		pars/command/record='sucint1' nd_0913_setup:setup.cnf -
			surreal7="''ecslope'"
$	endif
$	
$    endif
$!
$!   Stop acquisition and go back for more. 
$    ND_SA$POP_MSG "''txt_stopacq'1"
$    mca/command "off    mca0:[''username']det01"
$    wait 00:00:01
$    goto gainloop1
$!
$ endif   ! If the SET AMP/TCA boolean was not set
$!
$! Go back for more adjustments on this channel. 
$ goto gainloop2
$!
$finished:
$!
$ exit
$!
$!------------------------------------------------------------------------
$! Main error handling point. 
$ handle_error:
$    status=$status
$    statstr=f$message(status)
$    proc=f$environment("PROCEDURE")
$    procname=f$parse(proc,,,"NAME")
$    ND_SA$POP_MSG_ACKN "''procname' ''statstr'"
$    exit 1
$!
