$ on warning then goto handle_error
$ on control_y then goto handle_error
$!
$!****************************************************************************
$!
$!	48-0913 AMP/TCA 2016 Module Checking Procedure
$!
$! This procedure is used to perform a check for valid Amp/TCA configurations.
$!
$! Revision History:
$!
$!	25-NOV-1996	ARW	Original
$!
$!****************************************************************************
$!
$! If this routine was already called, then simply return. 
$ if moduleschecked .ne. 0 then exit
$!
$! See if detector one exists, if not show the user an error message.   
$ if f$trnlnm("ACQSTATE_''username'_DET01","ND_MCA_SYSTEM_TABLE") .eqs. ""
$ then 
$    ND_SA$POP_MSG_ACKN "''txt_det1notavail'"
$    exit
$ endif
$!
$! Let the user know we are checking the hardware modules. 
$ ND_SA$POP_MSG "''txt_chkmodules'"
$!
$! This is the AMP/TCA model number (stored in the device).
$ amptcamodel = 2
$!
$! Set up some state symbols. 
$ firsthdwerror  == 0
$ lastgoodmodule == 0
$ serialerrors   == 0
$!
$!---------------------------------------------------------------------------
$!
$! Get the AIM name for the detector mca configuration. 
$ pars/command/read mca0:['username']det01 cpname
$ aim1name = f$extract(0,f$locate(":",cpname),cpname)
$!
$! Now check out the AMP/TCAs connected to the first input. 
$ i = 1
$ loop1:
$!
$!   Check each icb input on the aim to see if it is an AMP/TCA. 
$    icbnum = i - 1
$    set noon
$!       Read the low part of the model number. 
$        mca/icb 'aim1name':'f$fao("!1XL",icbnum)' 0=4 1/sym=reg1 0/sym=reg0
$        status = $status
$    set on
$    if (.not. status) 
$    then 
$       if firsthdwerror .eq. 0 then firsthdwerror == i
$       goto next1
$    endif
$!
$!   If this is not a second generation of computer controlled NIM, 
$!   then simply skip this module (it can't be a 2016). 
$    if ((f$integer(reg0) .and. 3) .ne. 2) then goto next1
$!
$!   Calculate the low portion of the model number (in low four bits)
$    modello = reg1 .and. 15
$!
$!   Read the high part of the model number. 
$    mca/icb 'aim1name':'f$fao("!1XL",icbnum)' 0=32 1/sym=reg1
$!
$!   Calculate the high portion of the model number (in low four bits)
$    modelhi = reg1 .and. 15
$!
$!   Calculate the total model number. 
$    model = modello + (modelhi * 16)
$!
$!   Here we could check if the model number is an AmpTCAModel and 
$!   skip it if it is not the correct model number. 
$    if (model .ne. amptcamodel) 
$    then 
$       mca/icb 'aim1name':'f$fao("!1XL",icbnum)' 0=16	! Turn on Green LED 
$       goto next1
$    endif
$!
$!   Now get the serial number. 
$    serial = 0
$    j = 0
$    loop2:
$       newreg0 = 32 * (j + 2)
$       mca/icb 'aim1name':'f$fao("!1XL",icbnum)' 0="''newreg0'" 1/sym=reg1
$	serialnibble = reg1 .and. 15
$       call exponent 16 'j'
$	serial = serial + (serialnibble * exponent_result)
$       j = j + 1
$    if j .le. 5 then goto loop2
$!
$!   Store away the serial number to check for mismatches.
$    pars/command/read nd_0913_setup:setup.cnf surint1/rec='i'/entry=1
$    if (f$integer(surint1) .eq. 0) 
$    then 
$        pars/command nd_0913_setup:setup.cnf surint1='serial'/rec='i'/entry=1
$    else
$        if (f$integer(surint1) .ne. serial) then -
		 serialerrors == serialerrors + 1
$    endif
$!
$    lastgoodmodule == i
$    chnactive_'f$fao("!2ZL",i)' == 1
$!
$next1:
$    i = i + 1
$ if i .le. 16 then goto loop1
$!
$!---------------------------------------------------------------------------
$!
$! See if detector two exists, if not don't check the second set of inputs.
$ if f$trnlnm("ACQSTATE_''username'_DET02","ND_MCA_SYSTEM_TABLE") .eqs. ""
$ then 
$    goto done
$ endif
$!
$! For the second aim (inputs 17 to 32)
$!---------------------------------------------------------------------------
$!
$! Get the AIM name for the detector mca configuration. 
$ pars/command/read mca0:['username']det02 cpname
$ aim2name = f$extract(0,f$locate(":",cpname),cpname)
$!
$! Now check out the AMP/TCAs connected to the second input. 
$ i = 17
$ loop11:
$!
$!   Check each icb input on the aim to see if it is an AMP/TCA. 
$    icbnum = i - 17
$    set noon
$!       Read the low part of the model number. 
$        mca/icb 'aim2name':'f$fao("!1XL",icbnum)' 0=4 1/sym=reg1 0/sym=reg0
$        status = $status
$    set on
$    if (.not. status) 
$    then 
$       if firsthdwerror .eq. 0 then firsthdwerror == i
$       goto next11
$    endif
$! 
$!   If this is not a second generation of computer controlled NIM, 
$!   then simply skip this module (it can't be a 2016). 
$    if ((f$integer(reg0) .and. 3) .ne. 2) then goto next11
$!
$!   Calculate the low portion of the model number (in low four bits)
$    modello = reg1 .and. 15
$!
$!   Read the high part of the model number. 
$    mca/icb 'aim2name':'f$fao("!1XL",icbnum)' 0=32 1/sym=reg1
$!
$!   Calculate the high portion of the model number (in low four bits)
$    modelhi = reg1 .and. 15
$!
$!   Calculate the total model number. 
$    model = modello + (modelhi * 16)
$!
$!   Here we could check if the model number is an AmpTCAModel and 
$!   skip it if it is not the correct model number. 
$    if (model .ne. amptcamodel) 
$    then 
$       mca/icb 'aim2name':'f$fao("!1XL",icbnum)' 0=16	! Turn on Green LED 
$       goto next11
$    endif
$!
$!   Now get the serial number. 
$    serial = 0
$    j = 0
$    loop12:
$       newreg0 = 32 * (j + 2)
$       mca/icb 'aim2name':'f$fao("!1XL",icbnum)' 0="''newreg0'" 1/sym=reg1
$	serialnibble = reg1 .and. 15
$       call exponent 16 'j'
$	serial = serial + (serialnibble * exponent_result)
$       j = j + 1
$    if j .le. 5 then goto loop12
$!
$!   Store away the serial number to check for mismatches.
$    pars/command/read nd_0913_setup:setup.cnf surint1/rec='i'/entry=1
$    if (f$integer(surint1) .eq. 0) 
$    then 
$        pars/command nd_0913_setup:setup.cnf surint1='serial'/rec='i'/entry=1
$    else
$        if (f$integer(surint1) .ne. serial) then -
		 serialerrors == serialerrors + 1
$    endif
$!
$    lastgoodmodule == i
$    chnactive_'f$fao("!2ZL",i)' == 1
$!
$next11:
$    i = i + 1
$ if i .le. 32 then goto loop11
$!
$!---------------------------------------------------------------------------
$!
$done:
$!
$! Check if some modules are offline.  This assumes the modules are added 
$! in order, starting from 0, 1, 2, ....  If not a warning is given. 
$! if lastgoodmodule .gt. firsthdwerror 
$! then 
$!    ND_SA$POP_MSG_ACKN "''txt_moduleoffline'"
$! endif
$!
$! Check if there were any serial number errors. 
$ if serialerrors .gt. 0 
$ then 
$    ND_SA$POP_MSG_ACKN "''txt_serialnum'"
$ endif
$!
$! Set the global flag indicating that the modules were checked. 
$ moduleschecked == 1
$!
$finished:
$ exit
$!
$!------------------------------------------------------------------------
$! Main error handling exit point. 
$ handle_error:
$    status=$status
$    statstr=f$message(status)
$    proc=f$environment("PROCEDURE")
$    procname=f$parse(proc,,,"NAME")
$    ND_SA$POP_MSG_ACKN "''procname' ''statstr'"
$    exit 1
$!
$!========================================================================
$!
$exponent:
$ subroutine
$!
$ verflag = f$verify()
$ if verflag then set noverify
$!
$! Handle the case for an exponent of 0. 
$ exponent_result == 1
$ if p2 .le. 0 
$ then 
$    if verflag then set verify
$    exit 
$ endif
$!
$! Loop through, until the exponentiation is complete. 
$ ex = 1
$ exloop:
$    exponent_result == p1 * exponent_result  
$    ex = ex + 1
$ if ex .le. p2 then goto exloop
$!
$ if verflag then set verify
$ exit
$!
$ endsubroutine 
