$ on warning then goto handle_error
$!
$!****************************************************************************
$!
$!		2016 AMP/TCA Detector Maintenance Procedure
$!
$!			*****  ND_0913_DETECTOR.COM  *****
$!
$! This procedure is used to create, delete, and setup detector configurations.
$!
$!	Parameter Summary : 
$!		
$!		P1 - The operation type (CREATE, DELETE, SETUP)
$!
$!
$!	***********************  Version 1.0  ************************
$!
$!	Revision History:
$!
$!	10-Dec-1996	ARW	Original.  
$!
$!****************************************************************************
$!
$ labelname = "''p1'"
$!
$! Go to the correct operation point. 
$ goto det_'labelname'
$!
$!------------------------------------------------------------------------
$det_create:
$!
$! Create a detector configuration and setup file. 
$!
$! Determine a unique configuration name.
$! To do this, search the setup directory for the first file
$! of the form setup_det0#, where # is some number (either 1 or 2). 
$!
$ det_num = 1
$get_next:
$ name=f$search("nd_0913_setup:setup_det0''det_num'.cnf")
$ if name .eqs. "" then goto create_cnf
$ det_num = det_num + 1
$ goto get_next
$!
$! Create a detector configuration setup file
$create_cnf:
$!
$! Make sure the user has not exceeded the maximum number of sample changers.
$ if det_num .gt. 2 
$ then 
$     ND_SA$POP_MSG_ACKN "''txt_detlimit'"
$     exit
$ endif
$!
$! Create a temporary configuration for the detector configuration. 
$ define/nolog work nd_0913_setup:temporary
$ p1 = 4096
$ p2 = 1
$ p3 = "PHA"
$ run nd_sysexe:ndcrecnf
$!
$! Pars in the defaults. 
$ pars/command nd_0913_setup:temporary detname="det0''det_num'" - 
			detector="Detector ''det_num'" - 
			groups=1 cpname="pa:0"
$!
$! Ask the user for setup information, if he cancels delete the 
$! temporary configuration file and return. 
$!
$ ND_SA$PARS "nd_0913_setup:temporary nd_0913_text:det_setup.fpl"
$ if ND_SA_REPLY .eqs. CP_REPLY_CANCEL
$ then 
$   delete/nolog nd_0913_setup:temporary.cnf;*
$   exit
$ endif
$!
$! Reset the global so that the next time an operation is attempted, the 
$! modules will be rechecked. 
$ moduleschecked == 0
$!
$! Try to create the MCA
$ rename nd_0913_setup:temporary.cnf nd_0913_setup:setup_det0'det_num'.cnf
$ pars/command/read nd_0913_setup:setup_det0'det_num'.cnf cpname detname
$ set noon
$ mca create ['username']'detname' 'cpname'  -
	/default=nd_0913_setup:setup_det0'det_num'.cnf -
 	/prot=g:rwed
$ status = $status
$ set on
$ if .not. status
$ then
$!  
$!  If an error occurred during MCA creation, pop up a message and exit
$!
$   ND_SA$POP_MSG_ACKN "''txt_credet_err'
''f$message(status)'"
$   delete nd_0913_setup:setup_det0'det_num'.cnf;*
$   exit
$!
$ endif
$!
$! Let the user know that we are done. 
$ ND_SA$POP_MSG_ACKN "''txt_credone'"
$!
$ exit
$!
$!------------------------------------------------------------------------
$det_delete:
$!
$! Delete a detector configuration and setup file. 
$!
$! Get the name of the file to delete, try 2 first, then 1. 
$ number = 2
$ name=f$search("nd_0913_setup:setup_det02.cnf")
$ if name .nes. "" then goto delete_cnf
$!
$ number = 1
$ name=f$search("nd_0913_setup:setup_det01.cnf")
$ if name .nes. "" then goto delete_cnf
$!
$! If it reached here, then no detector configurations exist. 
$ ND_SA$POP_MSG_ACKN "''txt_nodets'"
$ exit
$!
$delete_cnf:
$!
$! Make absolutely sure the user wants to delete the detector
$! configuration.
$ ND_SA$POP_CONFIRM "''txt_delsure' ''number' ?|''txt_yesno'"
$ if (ND_SA_REPLY .eqs. CP_REPLY_CANCEL) then exit
$!
$! Reset the global so that the next time an operation is attempted, the 
$! modules will be rechecked. 
$ moduleschecked == 0
$!
$! Let the user know something is happening.
$ ND_SA$POP_MSG "''txt_working'"
$!
$! Get rid of the MVC in case the channel is open. 
$ set noon
$    mca delete/display 1 ! Delete it in case the channel is open
$    wait 00:00:01
$ set on
$!
$! Delete the detector mca configuration.
$ if f$trnlnm("ACQSTATE_''username'_DET0''number'","ND_MCA_SYSTEM_TABLE").nes.""
$ then
$   mca off mca0:['username']det0'number'     ! Make sure acquisition is off
$   wait 00:00:02               	    ! wait and delete it
$   mca delete mca0:['username']det0'number'
$ endif
$!
$! And delete the setup configuration file. 
$ delete/nolog nd_0913_setup:setup_det0'number'.cnf;*
$!
$! Let the user know that we are done. 
$ ND_SA$POP_MSG_ACKN "''txt_deldone'"
$!
$ exit
$!
$!------------------------------------------------------------------------
$det_setup:
$!
$! Perform automatic setup of detectors for all setup files present. 
$!
$ moduleschecked == 0
$!
$! Attempt to create detector number 1. 
$ name=f$search("nd_0913_setup:setup_det01.cnf")
$ if name .eqs. "" then goto next_cnf
$!
$ if f$trnlnm("ACQSTATE_''username'_DET01","ND_MCA_SYSTEM_TABLE").eqs.""
$ then
$!
$    pars/command/read nd_0913_setup:setup_det01.cnf cpname
$    set noon
$         mca create ['username']det01 'cpname' -
		/default=nd_0913_setup:setup_det01 -
		/prot=g:rwed
$	  status = $status
$    set on
$    if .not. status
$    then
$!	
$!	An error occurred during MCA create
$	ND_SA$POP_MSG_ACKN "''txt_mcacreerr' 1"
$!
$    endif
$!
$ endif
$!
$ next_cnf:
$!
$! Attempt to create detector number 2. 
$ name=f$search("nd_0913_setup:setup_det02.cnf")
$ if name .eqs. "" then goto done_cnf
$!
$ if f$trnlnm("ACQSTATE_''username'_DET02","ND_MCA_SYSTEM_TABLE").eqs.""
$ then
$!
$    pars/command/read nd_0913_setup:setup_det02.cnf cpname
$    set noon
$         mca create ['username']det02 'cpname' -
		/default=nd_0913_setup:setup_det02 -
		/prot=g:rwed
$	  status = $status
$    set on
$    if .not. status
$    then
$!	
$!	An error occurred during MCA create
$	ND_SA$POP_MSG_ACKN "''txt_mcacreerr' 2"
$!
$    endif
$!
$ endif
$!
$done_cnf:
$!
$ exit
$!
$!------------------------------------------------------------------------
$! Main error handling exit point. 
$ handle_error:
$    status=$status
$    statstr=f$message(status)
$    proc=f$environment("PROCEDURE")
$    procname=f$parse(proc,,,"NAME")
$    ND_SA$POP_MSG_ACKN "''procname' ''statstr'"
$    exit 1
$!
