$ on warning then goto handle_error
$ on control_y then goto handle_error
$!
$!****************************************************************************
$!
$!	48-0913 AMP/TCA 2016 Gain Normalization Procedure
$!
$! This procedure is used to perform gain normalization on the 2016
$! module. 
$!
$! Revision History:
$!
$!	02-Jan-1997	ARW	Original
$!
$!****************************************************************************
$!
$! Initialize the last tca module information that was selected variables. 
$ last_channel   == "0"
$ last_reg2value ==  0
$ last_icbnum    ==  0
$ last_aimname   ==  0
$!
$! See if detector one exists, if not show the user an error message.
$ if f$trnlnm("ACQSTATE_''username'_DET01","ND_MCA_SYSTEM_TABLE") .eqs. ""
$ then
$    ND_SA$POP_MSG_ACKN "''txt_det1notavail'"
$    goto finished
$ endif
$!
$! Call the routine to check out the hardware modules. 
$ @nd_0913_com:nd_0913_checkmods.com 
$! 
$! Clear the cc nim errors. 
$ pars/command mca0:['username']det01 astfbadcal=0 astfccerr=0 ! astfccfemod=0
$!
$! Put up the MVC window if it is not already up. 
$ ND_SA$CREATE_DISPLAY 1
$!
$! Clear the energy calibration information for detector one. 
$ pars/command mca0:['username']det01 -
        ecoffset="0" -
        ecslope="0" -
        ecalterms="0" -
        ecaltype="" -
        ecalunits="" -
        ecalcnv="0"
$!
$! Display detector one on the mca view control.
$ set noon
$    display/config mca0:['username']det01
$ set on
$!
$! Do the same for detector two if it exists. 
$ if f$trnlnm("ACQSTATE_''username'_DET02","ND_MCA_SYSTEM_TABLE") .nes. ""
$ then 
$   pars/command mca0:['username']det02 astfbadcal=0 astfccerr=0 ! astfccfemod=0
$   pars/command mca0:['username']det02 -
        ecoffset="0" -
        ecslope="0" -
        ecalterms="0" -
        ecaltype="" -
        ecalunits="" -
        ecalcnv="0"
$ endif
$!
$! If the channel number is not set yet, then default to 1. 
$ pars/command/read nd_0913_setup:setup sucint1
$ if (f$integer(sucint1) .lt. 1) .or. (f$integer(sucint1) .gt. 32)
$ then 
$    pars/command nd_0913_setup:setup sucint1=1  
$ endif
$!
$normloop1:
$!
$! Deselect the last TCA that was selected so that only one tca is ever
$! selected at a particular time. 
$  if (last_channel .nes. "0") 
$  then
$!
$!     Deselect the last channel. 
$      mca/icb 'last_aimname':'f$fao("!1XL",last_icbnum)' 2/sym=cur_reg2
$      cur_reg2 = f$integer(cur_reg2) .and. 127
$      mca/icb 'last_aimname':'f$fao("!1XL",last_icbnum)' 2="''cur_reg2'"
$!
$  endif
$!
$! Pop up the dialog to allow the user to select the channel. 
$  ND_SA$PARS "nd_0913_setup:setup nd_0913_text:channel_choice.fpl"
$!
$! If the user pressed cancel, then simply return. 
$ if ND_SA_REPLY .eqs. CP_REPLY_CANCEL then goto finished
$!
$! Get the user's choice of channel. 
$ pars/command/read nd_0913_setup:setup.cnf sucint1
$!
$! See if this channel is active or not. 
$ channel = f$integer(sucint1)
$ if (chnactive_'f$fao("!2ZL",channel)' .eq. 0)
$ then 
$    ND_SA$POP_MSG_ACKN "''txt_offline'"
$    goto normloop1
$ endif
$!
$! Open the proper input (1 or 2) 
$ newinputnum == ((channel - 1) / 16) + 1 
$!
$! Show the reference channel selection dialog. 
$  ND_SA$PARS "nd_0913_setup:setup nd_0913_text:channel_choice1.fpl"
$!
$! If the user pressed cancel, then go back for another channel. 
$ if ND_SA_REPLY .eqs. CP_REPLY_CANCEL then goto normloop1
$!
$! Get the reference channel number. 
$ pars/command/read nd_0913_setup:setup.cnf -
$		sucint3
$!
$! Get the corresponding energy calibration slope for this channel. 
$ pars/command/read/record='sucint3' nd_0913_setup:setup.cnf -
		surreal7
$ if (f$edit(surreal7,"TRIM,COMPRESS,UPCASE").eqs."0.00")
$ then 
$!   The selected channel is not calibrated yet. 
$    ND_SA$POP_MSG_ACKN "''txt_nocalib'"
$    goto normloop1
$ endif
$!
$! Set up the calibration in the detector configuration. 
$ pars/command mca0:['username']det01 -
        ecoffset="0" -
        ecslope="''surreal7'" -
        ecalterms="2" -
        ecaltype="Poly" -
        ecalunits="keV" -
        ecalcnv="1.0"
$!
$! Program the selected TCA (this sets the green led on). 
$ @nd_0913_com:nd_0913_progtca.com 'newinputnum' 'sucint1' 1 1 0
$!
$! Assert the select TCA line for the channel chosen.
$ if (f$integer(newinputnum) .eq. 1)
$ then
$    icbnum = f$integer(channel) - 1
$ else
$    icbnum = f$integer(channel) - 17
$ endif
$ pars/command/read mca0:['username']det0'newinputnum' cpname
$ aimname = f$extract(0,f$locate(":",cpname),cpname)
$ mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2/sym=new_reg2val
$ select_on = f$integer(new_reg2val) .or. 128
$ mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2="''select_on'"
$!
$! Enable the first SCA, make the window maximum, and start acquisition. 
$ lolld    = "1"
$ hilldreg = "0"
$ lould    = "255"
$ hiuldreg = "47"
$ mca/icb 'aimname':'f$fao("!1XL",icbnum)' -
                3="''lolld'" -
                4="''hilldreg'" -
                3="''lould'" -
                4="''hiuldreg'" -
		2/sym=reg2
$ newreg2val = f$integer(reg2) .or. 16		! enable first gate
$ mca/icb 'aimname':'f$fao("!1XL",icbnum)' -
		2="''newreg2val'"
$!
$! Start acquisition on the selected mca configuration. 
$ pars/command mca0:['username']det01 plive=0 preal=0
$ mca/command "erase mca0:[''username']det01"
$ mca/command "on    mca0:[''username']det01"
$!
$! Wait for the user to complete normalization. 
$ ND_SA$POP_MSG_ACKN "''txt_waitnorm'"
$ mca/command "off    mca0:[''username']det01"
$ wait 00:00:01
$ goto normloop1
$!
$finished:
$!
$ exit
$!
$!------------------------------------------------------------------------
$! Main error handling point. 
$ handle_error:
$    status=$status
$    statstr=f$message(status)
$    proc=f$environment("PROCEDURE")
$    procname=f$parse(proc,,,"NAME")
$    ND_SA$POP_MSG_ACKN "''procname' ''statstr'"
$    exit 1
$!
