$ on warning then goto handle_error
$ on control_y then goto handle_error
$!
$!****************************************************************************
$!
$!	48-0913 AMP/TCA 2016 Programming Routine
$!
$! This procedure is used to program the AMP/TCA based on the setup 
$! information for the given channel. 
$!
$! Arguments:
$!
$!	p1 - The detector number to setup  (1 or 2)
$!	p2 - The channel number to setup (1 to 32)
$!	p3 - program the SCA flag (0 or 1)
$!	p4 - program the AMP flag (0 or 1)
$!	p5 - apply all flag (0 or 1)
$! 
$! Returns:
$!    
$!      ND_PROGTCASTATUS - SUCCESS or ERROR
$!
$! Revision History:
$!
$!	07-DEC-1996	ARW	Original
$!
$!	30-DEC-1996 	ARW	Changed logic of the PUR Auto Threshold bit, 
$!				now, 0 allows adjustment not 1.  Changed 
$!				surflag2 to surflag3 in the calculations. 
$!
$!****************************************************************************
$!
$! Set up some local symbols. 
$ detnum = "''p1'"
$ channel = "''p2'"
$ progscaflag = f$integer(p3)
$ progampflag = f$integer(p4)
$ applyallflag = f$integer(p5)
$ nd_tca_calcscareg :== $nd_sysexe:nd_tca_calcscareg.exe
$!
$! Set the global return value defaults. 
$ ND_PROGTCASTATUS == "SUCCESS"
$!
$! Initialize the last tca module information that was selected, variables.
$ last_channel   == "0"
$ last_reg2value ==  0
$ last_icbnum    ==  0
$ last_aimname   ==  0
$!
$! If this channel is not active, then simply return. 
$ chan_number = f$integer(channel)
$ if (chnactive_'f$fao("!2ZL",chan_number)' .eq. 0)
$ then
$    exit
$ endif
$!
$! Get the icb number. 
$ if (f$integer(detnum) .eq. 1)
$ then 
$    icbnum = f$integer(channel) - 1
$ else
$    icbnum = f$integer(channel) - 17
$ endif
$!
$! Let the user know that the programming is occuring.
$! ND_SA$POP_MSG "''txt_progtca' ''channel'"
$!
$! Get the energy range set by the user for this channel.
$ pars/command/read/record='channel' nd_0913_setup:setup.cnf surint2
$!
$! Get some info from this mca configuration. 
$ pars/command/read mca0:['username']det0'detnum' cpname
$ aimname = f$extract(0,f$locate(":",cpname),cpname)
$!
$! Set up the AMP/TCA general settings and enable SCAs
$ pars/command/read/record='channel' nd_0913_setup:setup.cnf -
	surflag0 - 
	surflag1 - 
	surflag2 - 
	surflag3 - 
	surflag4  
$ pars/command/read/record='channel' nd_0913_setup:setup.cnf -
	surflag5 - 
	surflag6 -
	surint3 - 
	surint4
$!
$! Change all values to integer symbols. 
$ surflag0 = f$integer(surflag0)
$ surflag1 = f$integer(surflag1)
$ surflag2 = f$integer(surflag2)
$ surflag3 = f$integer(surflag3)
$ surflag4 = f$integer(surflag4)
$ surflag5 = f$integer(surflag5)
$ surflag6 = f$integer(surflag6)
$ surint4  = f$integer(surint4)
$! 
$! If the program amp flag is set, then do it. 
$ if progampflag 
$ then 
$!
$    reg2value = surint4 + (2 * surflag0) + (4 * surflag3)
$    reg2value = reg2value + (16 * surflag4) + (32 * surflag5) + (64 * surflag6)
$    mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2="''reg2value'"
$!
$!   Store the values for the last TCA module. 
$    last_channel   == channel
$    last_reg2value == reg2value
$    last_icbnum    == icbnum
$    last_aimname   == aimname
$!
$ endif
$!
$! If the program sca flag is set, then do it. 
$ if progscaflag
$ then 
$!
$!   Loop through and set the registers for all three windows. 
$    i = 1
$    scaloop:
$!
$!       Call the program to calculate the register values. 
$        nd_tca_calcscareg nd_0913_setup:setup.cnf 'channel' 'i'
$!
$!       Set the icb registers for this module. 
$        mca/icb 'aimname':'f$fao("!1XL",icbnum)' -
		3="''lolld'" - 
		4="''hilldreg'" - 
		3="''lould'" - 
		4="''hiuldreg'"
$!
$    i = i + 1
$    if i .le. 3 then goto scaloop
$!
$!   Turn LED to green signifying programming success
$    mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2/sym=reg2
$    newval = f$integer(reg2) .or. 8
$    mca/icb 'aimname':'f$fao("!1XL",icbnum)' 2="''newval'"
$!
$ endif
$!
$!-------------------------------------------------------------------------
$! If the apply all flag is set then do it for all. 
$ if applyallflag
$ then 
$!
$!   Let the user know what is happening. 
$!    ND_SA$POP_MSG "''txt_storeall'"
$!
$!   First get the values for this channel, to put into place for all 
$!   of the other modules. 
$    pars/command/read/record='channel' nd_0913_setup:setup.cnf -
		surint2 - 
		surint3 - 
		surint4 - 
		surflag0 - 
		surflag1 - 
		surflag2 
$    pars/command/read/record='channel' nd_0913_setup:setup.cnf -
		surflag3 -
		surflag4 - 
		surflag5 - 
		surflag6 - 
		surreal1 -
		surreal2
$    pars/command/read/record='channel' nd_0913_setup:setup.cnf -
		surreal3 - 
		surreal4 - 
		surreal5 - 
		surreal6
$!
$!   First, update the setup data file with all the data for all modules. 
$    i = 1
$    write_loop:
$!
$        ND_SA$POP_MSG "''txt_storeone' ''i'"
$!
$	 if (i .eq. f$integer(channel)) then goto next_write_loop 
$	 ND_SA$CAM_PUTP "nd_0913_setup:setup.cnf|''i'|1|surint2=''surint2'|surint3=''surint3'|surint4=''surint4'|surflag0=''surflag0'|surflag1=''surflag1'|surflag2=''surflag2'"
$	 ND_SA$CAM_PUTP "nd_0913_setup:setup.cnf|''i'|1|surflag3=''surflag3'|surflag4=''surflag4'|surflag5=''surflag5'|surflag6=''surflag6'|surreal1=''surreal1'|surreal2=''surreal2'"
$	 ND_SA$CAM_PUTP "nd_0913_setup:setup.cnf|''i'|1|surreal3=''surreal3'|surreal4=''surreal4'|surreal5=''surreal5'|surreal6=''surreal6'"
$!
$    next_write_loop:
$    i = i + 1
$    if i .le. 32 then goto write_loop
$!
$!   Restore the flags to integer values. 
$    surflag0 = f$integer(surflag0)
$    surflag1 = f$integer(surflag1)
$    surflag2 = f$integer(surflag2)
$    surflag3 = f$integer(surflag3)
$    surflag4 = f$integer(surflag4)
$    surflag5 = f$integer(surflag5)
$    surflag6 = f$integer(surflag6)
$    surint4  = f$integer(surint4)
$!
$!   Get the energy calibration for detector 1. 
$    pars/command/read mca0:['username']det01  cpname
$    aimname = f$extract(0,f$locate(":",cpname),cpname)
$!
$!   Now update all the AMP/TCA modules for detector 1. 
$    i = 1
$    settca_loop:
$!
$!	Skip the channel that was already set 
$	if (i .eq. f$integer(channel)) then goto next_settca_loop 
$!
$!      Skip modules that are not active.
$       if (chnactive_'f$fao("!2ZL",i)' .eq. 0) then goto next_settca_loop
$!
$!      Tell the user which one is being updated. 
$       ND_SA$POP_MSG "''txt_progtca' ''i'"
$!
$!      Set up the AMP/TCA general settings and enable SCAs. 
$       if progampflag 
$       then 
$!
$           reg2value = surint4 + (2 * surflag0) + (4 * surflag3)
$           reg2value = reg2value + (16 * surflag4) + (32 * surflag5) + (64 * surflag6) 
$           mca/icb 'aimname':'f$fao("!1XL",i-1)' 2="''reg2value'"
$!
$       endif
$!
$!      If the program sca flag is set, then do it. 
$       if progscaflag
$       then 
$!
$!         Loop through and set the registers for all three windows. 
$          ix = 1
$          scaloopa:
$!
$!             Call the program to calculate the register values. 
$              nd_tca_calcscareg nd_0913_setup:setup.cnf 'i' 'ix'
$!
$!             Set the icb registers for this module. 
$              mca/icb 'aimname':'f$fao("!1XL",i-1)' -
		      3="''lolld'" - 
		      4="''hilldreg'" - 
		      3="''lould'" - 
		      4="''hiuldreg'"
$!
$          ix = ix + 1
$          if ix .le. 3 then goto scaloopa
$!
$!	   Turn LED to green signifying programming success
$          mca/icb 'aimname':'f$fao("!1XL",i-1)' 2/sym=reg2
$          newval = f$integer(reg2) .or. 8
$          mca/icb 'aimname':'f$fao("!1XL",i-1)' 2="''newval'"
$!
$       endif	! end of if progscaflag
$!
$    next_settca_loop:
$    i = i + 1
$    if i .le. 16 then goto settca_loop
$!
$!   If detector two doesn't exist, then simply return. 
$    if f$trnlnm("ACQSTATE_''username'_DET02","ND_MCA_SYSTEM_TABLE") .eqs. ""
$    then
$	exit
$    endif
$!
$!   Get the energy calibration for detector 2. 
$    pars/command/read mca0:['username']det02  cpname
$    aimname = f$extract(0,f$locate(":",cpname),cpname)
$!
$!   +++++++++++++++++++++++++++++++++++++++++++++++++
$!   Now update all the AMP/TCA modules for detector 2. 
$    i = 17
$    settca_loopa:
$!
$!	Skip the channel that was already set 
$	if (i .eq. f$integer(channel)) then goto next_settca_loopa 
$!
$!      Skip modules that are not active.
$       if (chnactive_'f$fao("!2ZL",i)' .eq. 0) then goto next_settca_loopa
$!
$!      Tell the user which one is being updated. 
$       ND_SA$POP_MSG "''txt_progtca' ''i'"
$!
$!      Set up the AMP/TCA general settings and enable SCAs. 
$       if progampflag 
$       then 
$!
$           reg2value = surint4 + (2 * surflag0) + (4 * surflag3)
$           reg2value = reg2value + (16 * surflag4) + (32 * surflag5) + (64 * surflag6) 
$           mca/icb 'aimname':'f$fao("!1XL",i-17)' 2="''reg2value'"
$!
$       endif
$!
$!      If the program sca flag is set, then do it. 
$       if progscaflag
$       then 
$!
$!         Loop through and set the registers for all three windows. 
$          ix = 1
$          scaloopb:
$!
$!             Call the program to calculate the register values. 
$              nd_tca_calcscareg nd_0913_setup:setup.cnf 'i' 'ix'
$!
$!             Set the icb registers for this module. 
$              mca/icb 'aimname':'f$fao("!1XL",i-17)' -
		      3="''lolld'" - 
		      4="''hilldreg'" - 
		      3="''lould'" - 
		      4="''hiuldreg'"
$!
$          ix = ix + 1
$          if ix .le. 3 then goto scaloopb
$!
$!	   Turn LED to green signifying programming success
$          mca/icb 'aimname':'f$fao("!1XL",i-17)' 2/sym=reg2
$          newval = f$integer(reg2) .or. 8
$          mca/icb 'aimname':'f$fao("!1XL",i-17)' 2="''newval'"
$!
$       endif	! end of if progscaflag
$!
$    next_settca_loopa:
$    i = i + 1
$    if i .le. 32 then goto settca_loopa
$!
$ endif 	! end of if apply all
$!-------------------------------------------------------------------------
$!
$finished:
$ exit
$!
$!=========================================================================
$! Main error handling exit point. 
$ handle_error:
$    status=$status
$    statstr=f$message(status)
$    proc=f$environment("PROCEDURE")
$    procname=f$parse(proc,,,"NAME")
$    ND_SA$POP_MSG_ACKN "''procname' ''statstr'"
$    ND_PROGTCASTATUS == "ERROR"
$    exit 1
$!
