/************************************************************************
*
*		   TCA Energy Calibration Setting Program
*			Product # 48-0913
*
*	This program is used to set the energy calibration for an 
* 	mca configuration based on the user selected cursor channel
*	and the desired energy for the given channel.  A simple line
*	is forced through 0,0 and the chosen point. 
*
*	Calling Format:
*
*	ND_TCA_SETECAL <channel> <energy> 
*
*	Where:
*		<channel> is the channel number for the peak
*		<energy> is the energy for the peak
*
*************************************************************************
*
*	Revision History:
*
*	02-Jan-1997	ARW	Original version
*
************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <descrip.h>
#include <ndtypes.h>
#include <camdef.h>
#include <campardef.h>
#include <cammisc.h>
#include <cammsgdef.h>
#include <starlet.h>
#include <lib$routines.h>
#include <libclidef.h>

/* error handling macro */
#define IFERROR(status) if(!(status&1))

#define ECALTYPE    "Poly"
#define ECALUNITS   "keV"

/* local functions */
struct dsc$descriptor_s createDescrip(char *s);
int setSymbol(char *name, char *value);

/************************************************************************/
main(int argc, char *argv[])
{

LONG s;          			/* return status */
LONG channel = 0; 			/* channel number */
REAL energy = 0.0; 			/* energy range */
LONG ecalterms = 2; 			/* energy cal terms */
REAL ecoffset = 0.0; 			/* energy cal offset */
REAL ecslope;				/* energy cal slope */
REAL ecalcnv = 1.0; 			/* energy cal conversion factor */
char temp[256]; 			/* temporary string */

	/* make sure the program was called correctly */
	if (argc < 3)
	{
		fprintf(stdout, "Usage: ND_TCA_SETECAL <channel> <energy>\n"); 
		exit(1); 
	}

	/* get the channel passed in */
	channel = atol(argv[1]); 

	/* get the energy passed in */
	energy = atof(argv[2]); 

	/* calculate the energy calibration slope */
	if (channel > 0)
	{
		ecslope = energy / (float) channel; 
	}
	else
	{
		ecslope = 0.0; 
	}

	/* set up the symbols for use by the command procedure */
	sprintf(temp, "%.6g", ecoffset); 
	s = setSymbol("ECOFFSET", temp); 
	IFERROR(s) exit(s); 

	sprintf(temp, "%.6g", ecslope); 
	s = setSymbol("ECSLOPE", temp); 
	IFERROR(s) exit(s); 
	
	sprintf(temp, "%d", ecalterms); 
	s = setSymbol("ECALTERMS", temp); 
	IFERROR(s) exit(s); 

	sprintf(temp, "%s", ECALTYPE); 
	s = setSymbol("ECALTYPE", temp); 
	IFERROR(s) exit(s); 

	sprintf(temp, "%s", ECALUNITS); 
	s = setSymbol("ECALUNITS", temp); 
	IFERROR(s) exit(s); 

	sprintf(temp, "%.6g", ecalcnv); 
	s = setSymbol("ECALCNV", temp); 
	IFERROR(s) exit(s); 
	
	/* return success */
	return(1); 
} 

/************************************************************************
*
* Function: setSymbol
*
* Description: 	This function will set an individual DCL symbol. 
*
* Usage:
*
*	s = setSymbol(name, value)
*
*	where:
*
*		s = return status
*
*		name = symbol name to set
*
*		value = string to set the symbol
*	
************************************************************************/

int setSymbol(char *name, char *value)
{

struct dsc$descriptor_s temp1; 
struct dsc$descriptor_s temp2; 
int table = LIB$K_CLI_LOCAL_SYM; 


	/* create descriptors for the name and value */
	temp1 = createDescrip(name); 
	temp2 = createDescrip(value); 

	/* call the function to set up the global symbol */
	return(lib$set_symbol(&temp1, &temp2, &table)); 

}

/************************************************************************
*
* Function: createDescrip
*
* Description: 	This function will return a string descriptor properly 
* 		initialized to point to the null terminated string that
*		is passed in as an argument. 
*
* Usage:
*
*	descrip = createDescrip(string)
*
*	where:
*
*		descrip = returned string descriptor structure
*       	string  = a character pointer to a null terminated string
*	
************************************************************************/

struct dsc$descriptor_s createDescrip(char *s)
{

struct dsc$descriptor_s temp; 

	temp.dsc$b_class = DSC$K_CLASS_S; 	/* string descriptor */
	temp.dsc$b_dtype  = DSC$K_DTYPE_T; 	/* text data type */
	temp.dsc$a_pointer = s; 		/* string address */
	temp.dsc$w_length = strlen(s); 		/* length of string */

	/* return the initialized string descriptor */
	return(temp); 

}

/* end of file */

